@@ -501,8 +501,6 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         scene = self.plot_widget.scene()
         self._create_drag_tooltip(scene)
         self._data = None  # Original Table as passed from widget to new_data before transformations
-        self.attr_x = None
-        self.attr_y = None
 
         self.replot = self.plot_widget.replot
         ScaleScatterPlotData.__init__(self)
@@ -616,8 +614,8 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if data is None or not data.is_sparse():
             return data
 
-        attrs = {self.attr_x,
-                 self.attr_y,
+        attrs = {self.shown_x,
+                 self.shown_y,
                  self.attr_color,
                  self.attr_shape,
                  self.attr_size,
@@ -625,13 +623,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         domain = data.domain
         all_attrs = domain.variables + domain.metas
         attrs = list(set(all_attrs) & attrs)
-        selected_data = data[:, attrs]
-        if sp.issparse(selected_data.X):
-            selected_data.X = selected_data.X.toarray()
-        if sp.issparse(selected_data.Y):
-            selected_data.Y = selected_data.Y.toarray()
-        if sp.issparse(selected_data.metas):
-            selected_data.metas = selected_data.metas.toarray()
+        selected_data = data[:, attrs].to_dense()
         return selected_data
 
     def _clear_plot_widget(self):
@@ -655,12 +647,6 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.set_axis_title("left", "")
 
     def update_data(self, attr_x, attr_y, reset_view=True):
-        self.attr_x = attr_x
-        self.attr_y = attr_y
-        if attr_x not in self.data.domain or attr_y not in self.data.domain:
-            data = self.sparse_to_dense()
-            self.set_data(data)
-
         self.master.Warning.missing_coords.clear()
         self.master.Information.missing_coords.clear()
         self._clear_plot_widget()
@@ -672,13 +658,16 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             yaxis.textWidth = 30
 
         self.shown_x, self.shown_y = attr_x, attr_y
+        if attr_x not in self.data.domain or attr_y not in self.data.domain:
+            data = self.sparse_to_dense()
+            self.set_data(data)
 
         if self.jittered_data is None or not len(self.jittered_data):
             self.valid_data = None
         else:
             index_x = self.domain.index(attr_x)
             index_y = self.domain.index(attr_y)
-            self.valid_data = self.get_valid_list([index_x, index_y])
+            self.valid_data = self.get_valid_list([attr_x, attr_y])
             if not np.any(self.valid_data):
                 self.valid_data = None
         if self.valid_data is None:
@@ -724,7 +713,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             self.plot_widget.addItem(self.density_img)
 
         self.data_indices = np.flatnonzero(self.valid_data)
-        if len(self.data_indices) != self.original_data.shape[1]:
+        if len(self.data_indices) != len(self.data):
             self.master.Information.missing_coords(
                 self.shown_x.name, self.shown_y.name)
 
@@ -799,14 +788,13 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def compute_sizes(self):
         self.master.Information.missing_size.clear()
-        size_index = self.get_size_index()
-        if size_index == -1:
+        if self.attr_size is None:
             size_data = np.full((self.n_points,), self.point_width,
                                 dtype=float)
         else:
             size_data = \
                 self.MinShapeSize + \
-                self.scaled_data[size_index, self.valid_data] * \
+                self.scaled_data.get_column_view(self.attr_size)[0][self.valid_data] * \
                 self.point_width
         nans = np.isnan(size_data)
         if np.any(nans):
@@ -824,16 +812,16 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             self.scatterplot_item.setSize(size_data)
             self.scatterplot_item_sel.setSize(size_data + SELECTION_WIDTH)
 
-    def get_color_index(self):
+    def get_color(self):
         if self.attr_color is None:
-            return -1
+            return None
         colors = self.attr_color.colors
         if self.attr_color.is_discrete:
             self.discrete_palette = ColorPaletteGenerator(
                 number_of_colors=len(colors), rgb_colors=colors)
         else:
             self.continuous_palette = ContinuousPaletteGenerator(*colors)
-        return self.domain.index(self.attr_color)
+        return self.attr_color
 
     def compute_colors_sel(self, keep_colors=False):
         if not keep_colors:
@@ -868,7 +856,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
     def compute_colors(self, keep_colors=False):
         if not keep_colors:
             self.pen_colors = self.brush_colors = None
-        color_index = self.get_color_index()
+        self.get_color()
 
         def make_pen(color, width):
             p = QPen(color, width)
@@ -880,7 +868,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             subset = np.array([ex.id in self.subset_indices
                                for ex in self.data[self.valid_data]])
 
-        if color_index == -1:  # same color
+        if self.attr_color is None:  # same color
             color = self.plot_widget.palette().color(OWPalette.Data)
             pen = [make_pen(color, 1.5)] * self.n_points
             if subset is not None:
@@ -892,8 +880,8 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                         * self.n_points
             return pen, brush
 
-        c_data = self.original_data[color_index, self.valid_data]
-        if self.domain[color_index].is_continuous:
+        c_data = self.data.get_column_view(self.attr_color)[0][self.valid_data]
+        if self.attr_color.is_continuous:
             if self.pen_colors is None:
                 self.scale = DiscretizedScale(np.nanmin(c_data), np.nanmax(c_data))
                 c_data -= self.scale.offset
@@ -1004,19 +992,18 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             for label, text in zip(self.labels, label_data):
                 label.setText(text, black)
 
-    def get_shape_index(self):
+    def get_shape(self):
         if self.attr_shape is None or \
                 len(self.attr_shape.values) > len(self.CurveSymbols):
-            return -1
-        return self.domain.index(self.attr_shape)
+            return None
+        return self.attr_shape
 
     def compute_symbols(self):
         self.master.Information.missing_shape.clear()
-        shape_index = self.get_shape_index()
-        if shape_index == -1:
+        if self.get_shape() is None:
             shape_data = self.CurveSymbols[np.zeros(self.n_points, dtype=int)]
         else:
-            shape_data = self.original_data[shape_index, self.valid_data]
+            shape_data = self.data.get_column_view(self.attr_shape)[0][self.valid_data]
             nans = np.isnan(shape_data)
             if np.any(nans):
                 shape_data[nans] = len(self.CurveSymbols) - 1
@@ -1068,16 +1055,14 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.update_legend()
 
     def make_color_legend(self):
-        color_index = self.get_color_index()
-        if color_index == -1:
+        if self.attr_color is None:
             return
-        color_var = self.domain[color_index]
-        use_shape = self.get_shape_index() == color_index
-        if color_var.is_discrete:
+        use_shape = self.get_shape() == self.get_color()
+        if self.attr_color.is_discrete:
             if not self.legend:
                 self.create_legend()
             palette = self.discrete_palette
-            for i, value in enumerate(color_var.values):
+            for i, value in enumerate(self.attr_color.values):
                 color = QColor(*palette.getRGB(i))
                 brush = color.lighter(self.DarkerValue)
                 self.legend.addItem(
@@ -1095,15 +1080,14 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             legend.setGeometry(label.boundingRect())
 
     def make_shape_legend(self):
-        shape_index = self.get_shape_index()
-        if shape_index == -1 or shape_index == self.get_color_index():
+        shape = self.get_shape()
+        if shape is None or shape == self.get_color():
             return
         if not self.legend:
             self.create_legend()
-        shape_var = self.domain[shape_index]
         color = QColor(0, 0, 0)
         color.setAlpha(self.alpha_value)
-        for i, value in enumerate(shape_var.values):
+        for i, value in enumerate(self.attr_shape.values):
             self.legend.addItem(
                 ScatterPlotItem(pen=color, brush=color, size=10,
                                 symbol=self.CurveSymbols[i]), escape(value))
@@ -1184,36 +1168,47 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if self.scatterplot_item is None:
             return False
 
+        domain = self.data.domain
+        PARTS = (("Class", "Classes", 4, domain.class_vars),
+                 ("Meta", "Metas", 4, domain.metas),
+                 ("Feature", "Features", 10, domain.attributes))
+
+        def format_val(var, point_data, bold=False):
+            text = escape('{} = {}'.format(var.name, point_data[var]))
+            if bold:
+                text = "<b>{}</b>".format(text)
+            return text
+
+        def show_part(point_data, singular, plural, max_shown, vars):
+            cols = [format_val(var, point_data)
+                    for var in vars[:max_shown + 2]
+                    if vars == domain.class_vars
+                    or var not in (self.shown_x, self.shown_y)][:max_shown]
+            if not cols:
+                return ""
+            n_vars = len(vars)
+            if n_vars > max_shown:
+                cols[-1] = "... and {} others".format(n_vars - max_shown + 1)
+            return \
+                "<br/><b>{}</b>:<br/>".format(singular if n_vars < 2
+                                              else plural) \
+                + "<br/>".join(cols)
+
+        def point_data(p):
+            point_data = self.data[p.data()]
+            text = "<br/>".join(
+                format_val(var, point_data, bold=self.tooltip_shows_all)
+                for var in (self.shown_x, self.shown_y))
+            if self.tooltip_shows_all:
+                text += "<br/>" + \
+                        "".join(show_part(point_data, *columns)
+                                for columns in PARTS)
+            return text
+
         act_pos = self.scatterplot_item.mapFromScene(event.scenePos())
         points = self.scatterplot_item.pointsAt(act_pos)
-        text = ""
         if len(points):
-            for i, p in enumerate(points):
-                index = p.data()
-                text += "Attributes:\n"
-                if self.tooltip_shows_all and \
-                        len(self.domain.attributes) < 30:
-                    text += "".join(
-                        '   {} = {}\n'.format(attr.name,
-                                              self.data[index][attr])
-                        for attr in self.domain.attributes)
-                else:
-                    text += '   {} = {}\n   {} = {}\n'.format(
-                        self.shown_x, self.data[index][self.shown_x],
-                        self.shown_y, self.data[index][self.shown_y])
-                    if self.tooltip_shows_all:
-                        text += "   ... and {} others\n\n".format(
-                            len(self.domain.attributes) - 2)
-                if self.domain.class_var:
-                    text += 'Class:\n   {} = {}\n'.format(
-                        self.domain.class_var.name,
-                        self.data[index][self.data.domain.class_var])
-                if i < len(points) - 1:
-                    text += '------------------\n'
-
-            text = ('<span style="white-space:pre">{}</span>'
-                    .format(escape(text)))
-
+            text = "<hr/>".join(point_data(point) for point in points)
             QToolTip.showText(event.screenPos(), text, widget=self.plot_widget)
             return True
         else:
@@ -1236,14 +1231,14 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def zoom_actions(self, parent):
         def zoom(s):
-            """Zoom in/out by factor `s`."""
-            viewbox = self.plot.getViewBox()
-            # scaleBy scales the view's bounds (the axis range)
-            viewbox.scaleBy((1 / s, 1 / s))
+            """
+            Zoom in/out by factor `s`.
+            scaleBy scales the view's bounds (the axis range)
+            """
+            self.view_box.scaleBy((1 / s, 1 / s))
 
         def fit_to_view():
-            viewbox = self.plot.getViewBox()
-            viewbox.autoRange()
+            self.viewbox.autoRange()
 
         zoom_in = QAction(
             "Zoom in", parent, triggered=lambda: zoom(1.25)
